@@ -79,6 +79,76 @@ ccc.nonpara.bootstrap <- function(X, nboot, alpha){
     CI <- as.vector(quantile(unlist(ccc.boot), c(alpha/2, 1-alpha/2), type=6))
     list(point=point, icl=CI[1], icu=CI[2])
 }
+
+#-----------------------------------------------------------------------------
+ccc.mvst.mcmc <- function(X, alpha, nmcmc=10000){
+
+    mcmc.mvst <- mvst.mcmc(X, nmcmc=nmcmc*1.1)
+    Mu <- mcmc.mvst$Mu
+    D <- mcmc.mvst$Delta
+    Sigma <- mcmc.mvst$Sigma
+    nu <- mcmc.mvst$nu
+    #DIC <- mcmc.mvst$DIC
+    
+    nmcmc <- nrow(Mu) 
+    k <- ncol(X)
+    ccc.Bayes <- rep(0, nmcmc)
+    for(iter in 1:nmcmc){
+        M <- Mu[iter,] + (nu[iter]/pi)^(1/2)*gamma((nu[iter]-1)/2) / gamma(nu[iter]/2) * D[iter,]
+        DD <- D[iter,] %*% t(D[iter,])
+        Sig <- (Sigma[iter,,] + diag(D[iter,])^2 + 2/pi*(DD-diag(D[iter,])^2)) * (nu[iter]/(nu[iter]-2)) -
+               nu[iter] / pi * (gamma((nu[iter]-1)/2) / gamma(nu[iter]/2))^2 * DD
+
+        ss <- 0
+        sm <- 0
+        for(i in 1:(k-1)){
+            for(j in (i+1):k){
+                ss <- ss + Sig[i,j]
+                sm <- sm + (M[i] - M[j])^2
+            }
+        }
+        ccc.Bayes[iter] <- 2*ss / ((k-1)*sum(diag(Sig)) + sm)
+    }
+    ccc.Bayes <- mcmc(ccc.Bayes)
+    hpd <- as.vector(HPDinterval(ccc.Bayes, 1-alpha))
+
+    list(point=median(ccc.Bayes), icl=hpd[1], icu=hpd[2])
+
+}
+#-----------------------------------------------------------------------------
+ccc.mvsn.mcmc <- function(X, alpha, nmcmc){
+
+    mcmc.mvsn <- mvsn.mcmc(X, nmcmc=nmcmc*1.1)
+    Mu <- mcmc.mvsn$Mu
+    Delta <- mcmc.mvsn$Delta
+    Sigma <- mcmc.mvsn$Sigma
+    #DIC <- mcmc.mvsn$DIC
+
+    nmcmc <- nrow(Mu) 
+    k <- ncol(X)
+    ccc.Bayes <- rep(0, nmcmc)
+    for(iter in 1:nmcmc){
+        Sig <- Sigma[iter,,] + (1-2/pi)*diag(Delta[iter,])^2
+        M <- Mu[iter,] + sqrt(2/pi)*Delta[iter,]
+        ss <- 0
+        sm <- 0
+        for(i in 1:(k-1)){
+            for(j in (i+1):k){
+                ss <- ss + Sig[i,j]
+                sm <- sm + (M[i] - M[j])^2
+            }
+        }
+        ccc.Bayes[iter] <- 2*ss / ((k-1)*sum(diag(Sig)) + sm)
+    }
+
+    ccc.Bayes <- mcmc(ccc.Bayes)
+    hpd <- as.vector(HPDinterval(ccc.Bayes, 1-alpha))
+
+    list(point=median(ccc.Bayes), icl=hpd[1], icu=hpd[2])
+
+}
+
+
 #-----------------------------------------------------------------------------
 ccc.lognormalNormal.mcmc <- function(X, alpha, nmcmc){
     mcmc.lognormalNormal <- lognormalNormal.mcmc(X=X, nmcmc=nmcmc)
@@ -168,7 +238,7 @@ agree.ccc <- function(ratings, conf.level=0.95,
                       method=c("jackknifeZ", "jackknife",
                                "bootstrap", "bootstrapBC",
                                "mvn.jeffreys", "mvn.conjugate",
-                               "mvt", "lognormalNormal"),
+                               "mvt", "lognormalNormal", "mvsn", "mvst"),
                       nboot=999, nmcmc=10000,
                       mvt.para=list(prior=list(lower.v=4, upper.v=25,
                                                Mu0=rep(0, ncol(ratings)),
@@ -197,12 +267,14 @@ agree.ccc <- function(ratings, conf.level=0.95,
     alpha <- 1 - conf.level
     
     estimate <- switch(method,
+                       mvsn = ccc.mvsn.mcmc(X, alpha, nmcmc),
+                       mvst = ccc.mvst.mcmc(X, alpha, nmcmc),
                        lognormalNormal = ccc.lognormalNormal.mcmc(X, alpha, nmcmc),
                        mvt = ccc.mvt.mcmc(X, alpha, nmcmc,
-                         mvt.para$prior$lower.v, mvt.para$prior$upper.v,
-                         mvt.para$prior$Mu0, mvt.para$prior$Sigma0,
-                         mvt.para$prior$p, mvt.para$prior$V,
-                         mvt.para$initial$v, mvt.para$initial$Sigma),
+                           mvt.para$prior$lower.v, mvt.para$prior$upper.v,
+                           mvt.para$prior$Mu0, mvt.para$prior$Sigma0,
+                           mvt.para$prior$p, mvt.para$prior$V,
+                           mvt.para$initial$v, mvt.para$initial$Sigma),
                        mvn.jeffreys = ccc.mvn.mcmc(X, alpha, nmcmc, "Jeffreys"),
                        mvn.conjugate = ccc.mvn.mcmc(X, alpha, nmcmc, "Conjugate"),
                        jackknifeZ = unlist(ccc.nonpara.jackknife(X, alpha)[c(2,4)]),
